@@ -32,10 +32,15 @@
  * 28 Dec 1999  Version 2.5.11
  *  4 Jan 2000  Version 2.5.12
  * 24 May 2000  Version 2.5.14
- *  9 Oct 2001  Version 2.7.1  Enhanced DVI update check
+ *  9 Oct 2001  Version 2.7.1  Enhanced DVI update check.
+ * 15 Dec 2001  Version 2.8.0  Enhanced DVI update check again. 
+ *                 Library header file name is changed ("libdvi28.h").
+ *  8 Oct 2002  Version 2.8.1  Fixed memory error.
+ * 18 Nov 2002  Version 2.9.0  Added user data area in dvi object.
+ * 2  May 2007  Version 2.9.1  Fixed memory leaks in dvi.c.
  */
 /*
- * Copyright (C) 1996-2001  Hirotsugu Kakugawa. 
+ * Copyright (C) 1996-2007  Hirotsugu Kakugawa. 
  * All rights reserved.
  *
  * This file is part of the DVIlib Library.  This library is free
@@ -54,6 +59,8 @@
 #include "../config.h"
 #include <stdio.h>
 #include <stdlib.h>
+#include <limits.h>
+#include <sys/param.h>
 #ifdef HAVE_SYS_TYPES_H
 #  include <sys/types.h>
 #endif
@@ -77,7 +84,7 @@
 #include <math.h>
 
 #include "VFlib-3_6.h"
-#include "dvi-2_6.h"
+#include "libdvi29.h"
 #include "defs.h"
 #include "cache.h"
 #include "private.h"
@@ -86,7 +93,7 @@
 #include "dviinst.h"
 
 
-#define  HASH_TABLE_SIZE  191
+#define  HASH_TABLE_SIZE  311
 #define  HASH(i)  ((i)%HASH_TABLE_SIZE)
 
 
@@ -130,7 +137,9 @@ struct s_cache_key {
   DVI_DEVICE  dev;
 };
 typedef struct s_cache_key  *CACHE_KEY;
+
 #define ENV_FONT_CACHE_SIZE  "DVILIB_FONT_CACHE_SIZE"
+
 Private int  font_cache_size = FONT_CACHE_SIZE;
 Private VF_BITMAP  font_load_bitmap(CACHE,CACHE_KEY,int);
 
@@ -144,8 +153,8 @@ Private VF_BITMAP  font_load_bitmap(CACHE,CACHE_KEY,int);
 #define MAG2IMAG(m)  ((m)*1000)
 
 Private int    altm_draw_ps_figures(DVI_DEVICE,DVI);
-Private void   altm_before_ps_figure(DVI_DEVICE,DVI,char*,long,long);
-Private void   altm_after_ps_figure(DVI_DEVICE,DVI);
+Private int    altm_before_ps_figure(DVI_DEVICE,DVI,char*,long,long,long);
+Private void   altm_after_ps_figure(DVI_DEVICE,DVI,char*,long,long,long);
 Private int    altm_ps_figure_display_step(DVI_DEVICE,DVI,long,long);
 #if 0
 Private int    altm_special_command_undef(DVI_DEVICE,DVI,char*,long,long);
@@ -164,6 +173,7 @@ Private char*  altm_temp_dir(DVI_DEVICE,DVI);
 Glocal char  *dvi_debug_flags = NULL;
 Glocal int    dvi_debug(char);
 
+
 #define SCALEBOX 1
 
 
@@ -208,7 +218,7 @@ DVI_init(char *vflibcap, char *vflib_params)
 
   font_cache_size = FONT_CACHE_SIZE;
   if ((s = getenv(ENV_FONT_CACHE_SIZE)) != NULL)
-    if ((font_cache_size = atoi(s)) < 2)
+    if ((font_cache_size = atoi(s)) < 5)
       font_cache_size = FONT_CACHE_SIZE;
   if (dvi_debug('C'))
     printf("DVIlib: Cache size = %d\n", font_cache_size);
@@ -246,10 +256,6 @@ DVI_create(DVI_DEVICE dev, char* path, DVI_PROPERTY props)
   DVI    dvi;
   PDATA  pd;
   char   *dvi_file_name = NULL;
-  int    f_ac;
-#ifdef HAVE_STAT 
-  struct stat  st;
-#endif
 
   if (dvi_inited_flag <= 0){
     if (DVI_init(NULL, NULL) < 0)
@@ -289,9 +295,22 @@ DVI_create(DVI_DEVICE dev, char* path, DVI_PROPERTY props)
     return NULL;
   }
 
+  dvi_file_name = DVI_find_dvi_path(path);
+  if (dvi_file_name == NULL){
+    free(dvi);
+    free(pd);
+    return NULL;
+  }
 
+#if 0
+  int    f_ac;
 #ifdef HAVE_STAT 
-# if defined(__linux__)
+  struct stat  st;
+#endif
+#endif
+#if 0
+#ifdef HAVE_STAT 
+# if defined(__linux__) || defined(__GLIBC__)
   f_ac = ((stat(path, &st) >= 0) && S_ISREG(st.st_mode));
 # else
   f_ac = ((stat(path, &st) >= 0) && ((st.st_mode & S_IFMT) == S_IFREG));
@@ -299,14 +318,14 @@ DVI_create(DVI_DEVICE dev, char* path, DVI_PROPERTY props)
 #else
   f_ac = (access(path, R_OK) >= 0);
 #endif
-
   if (f_ac) {
     STRDUP(dvi_file_name, path);
   } else {
-    if ((dvi_file_name = (char*)malloc(strlen(path)+5)) != NULL){
-      sprintf(dvi_file_name, "%s%s", path, ".dvi");
+    int  fnlen = strlen(path) + 16; 
+    if ((dvi_file_name = (char*)malloc(fnlen)) != NULL){
+      snprintf(dvi_file_name, fnlen, "%s%s", path, ".dvi");
       if (access(dvi_file_name, R_OK) != 0){
-	sprintf(dvi_file_name, "%s%s", path, "dvi");
+	snprintf(dvi_file_name, fnlen, "%s%s", path, "dvi");
 	if (access(dvi_file_name, R_OK) != 0){
 	  free(dvi_file_name);
 	  dvi_file_name = NULL;
@@ -314,11 +333,8 @@ DVI_create(DVI_DEVICE dev, char* path, DVI_PROPERTY props)
       }
     }
   }
-  if (dvi_file_name == NULL){
-    free(dvi);
-    free(pd);
-    return NULL;
-  }
+#endif
+
 
   dvi->open_font             = DVI_open_font;
   dvi->dispose               = DVI_dispose;
@@ -331,6 +347,10 @@ DVI_create(DVI_DEVICE dev, char* path, DVI_PROPERTY props)
   dvi->error                 = 0;
   dvi->job_name[0]           = '\0';
   dvi->privates              = (void*)pd;
+  dvi->udi1 = dvi->udi2 = dvi->udi3 = dvi->udi4 = 0;
+  dvi->udr1 = dvi->udr2 = dvi->udr3 = dvi->udr4 = 0;
+  dvi->udp1 = dvi->udp2 = dvi->udp3 = dvi->udp4 = NULL;
+  dvi->udp5 = dvi->udp6 = dvi->udp7 = dvi->udp8 = NULL;
 
   if (props == NULL)
     PD(dvi,property) = DVI_property_alloc_default();
@@ -380,6 +400,7 @@ DVI_create(DVI_DEVICE dev, char* path, DVI_PROPERTY props)
 		       (void(*)(void*))VF_FreeBitmap);
   if (dvi_color_init(dvi, dev) < 0)
     goto ERROR;
+  PD(dvi,special_pos) = -1;
   PD(dvi,gstate_stack_ptr) = 0;
   PD(dvi,gstate_transformed) = 0;
   PD(dvi,gstate_scale_h) = 1;
@@ -393,6 +414,10 @@ DVI_create(DVI_DEVICE dev, char* path, DVI_PROPERTY props)
 
   dvi_paper_size_in_dvi(dvi, dev);
 
+#if 0
+  printf("*** Created DVI: %s\n", path);
+#endif
+
   return dvi;
 
 
@@ -417,11 +442,24 @@ dvi_set_filestamp(DVI dvi, DVI_DEVICE dev)
   if (stat(PD(dvi,file_name), &st) < 0)
     return -1;
 
+#if   defined(HAVE_STAT_ST_MTIMESPEC_TV_SEC)
+  PD(dvi,file_timestamp1) = st.st_mtimespec.tv_sec;
+# if  defined(HAVE_STAT_ST_MTIMESPEC_TV_NSEC)
+  PD(dvi,file_timestamp2) = st.st_mtimespec.tv_nsec;
+# endif
+#elif defined(HAVE_STAT_ST_MTIME)
+  PD(dvi,file_timestamp1) = st.st_mtime;
+# if  defined(HAVE_STAT_ST_MTIMENSEC)
+  PD(dvi,file_timestamp2) = st.st_mtimensec;
+# endif
+#endif
+
+#if 0
 #ifndef _POSIX_SOURCE
 # if defined(__svr4__)
   PD(dvi,file_timestamp1) = st.st_mtime;
   PD(dvi,file_timestamp2) = 0;
-# elif defined(__linux__)
+# elif defined(__linux__) || defined(__GLIBC__)
   PD(dvi,file_timestamp1) = st.st_mtime;
   PD(dvi,file_timestamp2) = 0;
 # else
@@ -432,6 +470,7 @@ dvi_set_filestamp(DVI dvi, DVI_DEVICE dev)
   PD(dvi,file_timestamp1) = st.st_mtime;
   PD(dvi,file_timestamp2) = st.st_mtimensec;
 #endif
+#endif
 
 #endif /*HAVE_STAT*/
 
@@ -450,14 +489,33 @@ DVI_file_modified(DVI dvi, char *f)
   return 0;   /* not modified */
 #else
   struct stat  st;
+  time_t  t1 = 0;
+  time_t  t2 = 0;
 
   if (stat(PD(dvi,file_name), &st) < 0)
     return 1;   /* modified (probably, removed) */
+
+#if    defined(HAVE_STAT_ST_MTIMESPEC_TV_SEC)
+  t1 = st.st_mtimespec.tv_sec;
+# if   defined(HAVE_STAT_ST_MTIMESPEC_TV_NSEC)
+  t2 = st.st_mtimespec.tv_nsec;
+# endif
+#elif  defined(HAVE_STAT_ST_MTIME)
+  t1 = st.st_mtime;
+# if   defined(HAVE_STAT_ST_MTIMENSEC)
+  t2 = st.st_mtimensec;
+# endif
+#endif
+  if ((PD(dvi,file_timestamp1) != (long)t1) 
+      || (PD(dvi,file_timestamp2) != (long)t2))
+    return 1;   /* modified */
+
+#if 0
 #ifndef _POSIX_SOURCE
 # if defined(__svr4__)
   if (PD(dvi,file_timestamp1) != (long)st.st_mtime)
     return 1;   /* modified */
-# elif defined(__linux__)
+# elif defined(__linux__) || defined(__GLIBC__)
   if (PD(dvi,file_timestamp1) != (long)st.st_mtime)
     return 1;   /* modified */
 # else
@@ -470,8 +528,10 @@ DVI_file_modified(DVI dvi, char *f)
       || (PD(dvi,file_timestamp2) != (long)st.st_mtimensec))
     return 1;   /* modified */
 #endif
-  return 0;  /* not modified */
 #endif
+
+  return 0;  /* not modified */
+#endif  /* HAVE_STAT */
 }
 
 
@@ -543,6 +603,7 @@ read_dvi_file_info(DVI dvi, DVI_DEVICE dev, FILE* fp)
   long           ofs;
   
   /** Preamble **/
+
   b1 = READ_UINT1(fp);
   b2 = READ_UINT1(fp);
   if ((b1 != DVIINST_PRE) || (b2 != 2)){
@@ -562,8 +623,6 @@ read_dvi_file_info(DVI dvi, DVI_DEVICE dev, FILE* fp)
   
   i = 0;
   for (k1 = READ_UINT1(fp); k1 > 0; --k1){
-    if (((i%100)==0) && (DVI_file_modified(dvi, PD(dvi,file_name)) == 1))
-      return FALSE;
     dvi->job_name[i] = (char)READ_UINT1(fp);
     i++;
   }
@@ -571,16 +630,15 @@ read_dvi_file_info(DVI dvi, DVI_DEVICE dev, FILE* fp)
   PD(dvi,offset_first_page) = ftell(fp);
 
   /** Postamble **/
-  i = 0;
+
   ofs = 0L;
   do {
-    if (((i%100)==0) && (DVI_file_modified(dvi, PD(dvi,file_name)) == 1))
-      return FALSE;
     --ofs;
-    fseek(fp, ofs, SEEK_END);
+    if (fseek(fp, ofs, SEEK_END) < 0)
+      return FALSE;
     b1 = READ_UINT1(fp);
-    i++;
   } while (b1 == (UINT1)DVIINST_TRAILER);
+
   /* check post_post arg */
   dvi->ptex_dvi = 0;
   switch ((int)b1){
@@ -600,21 +658,26 @@ read_dvi_file_info(DVI dvi, DVI_DEVICE dev, FILE* fp)
 
   /* check POST_POST inst */
   ofs = ofs - 5L;
-  fseek(fp, ofs, SEEK_END);
+  if (fseek(fp, ofs, SEEK_END) < 0)
+    return FALSE;
   if (READ_UINT1(fp) != (UINT1)DVIINST_POSTPOST){
     dvi->error = DVI_ERR_ILL_FORMAT;
     return FALSE;
   }
+
   /* read pointer to POST inst */
   post_offset = (long)READ_INT4(fp);
 
   /** POST **/
-  fseek(fp, post_offset, SEEK_SET);
+
+  if (fseek(fp, post_offset, SEEK_SET) < 0)
+    return FALSE;
   if (READ_UINT1(fp) != (UINT1)DVIINST_POST){
     dvi->error = DVI_ERR_ILL_FORMAT;
     return FALSE;
   }
   PD(dvi,offset_last_page) = READ_UINT4(fp);
+
   p_num = READ_UINT4(fp);
   p_den = READ_UINT4(fp);
   p_mag = READ_UINT4(fp);
@@ -623,12 +686,16 @@ read_dvi_file_info(DVI dvi, DVI_DEVICE dev, FILE* fp)
     dvi->error = DVI_ERR_ILL_FORMAT;
     return FALSE;
   }
+
   PD(dvi,max_height) = READ_INT4(fp);
   PD(dvi,max_width)  = READ_INT4(fp);
   PD(dvi,max_stack)  = READ_UINT2(fp);
   dvi->pages       = READ_UINT2(fp);
   PD(dvi,offset_fontdef) = ftell(fp);
 
+  if (DVI_file_modified(dvi, PD(dvi,file_name)) == 1)
+    return FALSE;
+
   return TRUE;
 }
 
@@ -651,11 +718,15 @@ read_dvi_file_index(DVI dvi, DVI_DEVICE dev, FILE* fp)
   p = dvi->pages;
   PD(dvi,page_index)[p-1] = PD(dvi,offset_last_page);
   for ( ; p > 0; p--){
-    fseek(fp, PD(dvi,page_index)[p-1]+1, SEEK_SET);
+    if (fseek(fp, PD(dvi,page_index)[p-1]+1, SEEK_SET) < 0)
+      return FALSE;
     for (i = 0; i < 10; i++)
       dvi->page_no[p-1].c[i] = READ_UINT4(fp);
-    if (p >= 2)
+    if (p >= 2){
       PD(dvi,page_index)[p-2] = READ_UINT4(fp);
+      if (PD(dvi,offset_last_page) < PD(dvi,page_index)[p-2])
+	return FALSE;
+    }
     if (DEV_METHOD_DEF(dev,device_polling)){
       if (DEV_CALL(dev,device_polling)(dev, dvi, DVI_POLL_FILE) != 0)
 	return FALSE;
@@ -690,7 +761,10 @@ DVI_open_font(DVI dvi, DVI_DEVICE dev)
     dvi->error = DVI_ERR_INTERNAL;
     return -1;
   }
-  fseek(PD(dvi,dvi_file), PD(dvi,offset_fontdef), SEEK_SET);
+  if (fseek(PD(dvi,dvi_file), PD(dvi,offset_fontdef), SEEK_SET) < 0){
+    dvi->error = DVI_ERR_NOT_ACTIVE;
+    return -1;
+  }
 
   val = 0;
   for (;;){
@@ -785,7 +859,6 @@ DVI_dispose(DVI dvi, DVI_DEVICE dev)
   free_if_non_null(PD(dvi,color_background));
   free_if_non_null(dvi->privates);
 
-  PD(dvi,alive) = FALSE;
   free(dvi);
 
   return 0;
@@ -808,7 +881,10 @@ DVI_draw_page(DVI dvi, DVI_DEVICE dev, int page, double shrink)
     dvi->error = DVI_ERR_ILL_PAGE_NO;
     return -1;
   }
-  fseek(PD(dvi,dvi_file), PD(dvi,page_index)[page-1], SEEK_SET);
+  if (fseek(PD(dvi,dvi_file), PD(dvi,page_index)[page-1], SEEK_SET) < 0){
+    dvi->error = DVI_ERR_NOT_ACTIVE;
+    return -1;
+  }
 
   return dvi_page(dvi, dev, page, shrink);
 }
@@ -893,7 +969,10 @@ DVI_restore_context(DVI dvi, DVI_DEVICE dev, DVI_CONTEXT context)
   for (s = 0; s <= context->depth; s++)
     PD(dvi,stack)[s]  = context->stack[s];
   PD(dvi,stackp)      = context->depth;
-  fseek(PD(dvi,dvi_file), context->offset, SEEK_SET);
+  if (fseek(PD(dvi,dvi_file), context->offset, SEEK_SET) < 0){
+    dvi->error = DVI_ERR_NOT_ACTIVE;
+    return -1;
+  }
   dvi_color_stack_resume(dvi, dev, context->color_stack, context->color_bg);
   PD(dvi,gstate_stack_ptr)   = context->gstate_stack_ptr;
   PD(dvi,gstate_transformed) = context->gstate_transformed;
@@ -1072,7 +1151,7 @@ dvi_page(DVI dvi, DVI_DEVICE dev, int page, double shrink)
   double        m;
 
   dvi->current_page = page;
-  strcpy(dvi->curr_page_name, "");
+  strncpy(dvi->curr_page_name, "", sizeof(dvi->curr_page_name));
   PD(dvi,shrink) = shrink;
   PD(dvi,gstate_stack_ptr) = 0;
   PD(dvi,gstate_transformed) = 0;
@@ -1086,7 +1165,9 @@ dvi_page(DVI dvi, DVI_DEVICE dev, int page, double shrink)
       return DVI_DRAW_ERROR;
   }
 
+#if 1
   dvi_color_stack_restore_page(dvi, dev, page);
+#endif
 
   dvi->error = 0;
 
@@ -1096,13 +1177,12 @@ dvi_page(DVI dvi, DVI_DEVICE dev, int page, double shrink)
 
     if (DEV_METHOD_DEF(dev,device_polling)){
       if (DEV_CALL(dev,device_polling)(dev, dvi, DVI_POLL_PAGE) != 0){
-	dvi_color_reset_page(dvi, dev, NULL);
 	return DVI_DRAW_INTERRUPTED;
       }
     }
 
     instr = READ_UINT1(PD(dvi,dvi_file));
-    if (instr <= DVIINST_SET4){  /* SETCHAR0 ... SETCHAR127, SET1, ... ,SET4 */
+    if (instr <= DVIINST_SET4){  /* SETCHAR0..SETCHAR127, SET1..SET4 */
       if ((code = instr) > (DVIINST_SETCHAR0+127)){
 	n = instr - DVIINST_SET1 + 1;
 	code = (UINT4)READ_UINTN(PD(dvi,dvi_file), n);
@@ -1121,8 +1201,13 @@ dvi_page(DVI dvi, DVI_DEVICE dev, int page, double shrink)
 	PD(dvi,stackp) = 0;
 	HH = VV = WW = XX = YY = ZZ = 0L;
 	DD = 0L;  /* for pTeX, vertical writing feature */
-	sprintf(dvi->curr_page_name, "%ld", READ_INT4(PD(dvi,dvi_file)));
+	{
+	  char s[1024];
+	  snprintf(s, sizeof(s), "%ld", READ_INT4(PD(dvi,dvi_file)));
+	  strncpy(dvi->curr_page_name, s, sizeof(dvi->curr_page_name));
+	}
 	SKIP_N(PD(dvi,dvi_file), 4*10);
+	/*** BG COLOR ***/
 	break;
 
       case DVIINST_EOP:
@@ -1289,7 +1374,10 @@ dvi_page(DVI dvi, DVI_DEVICE dev, int page, double shrink)
 
       case DVIINST_XXX1: case DVIINST_XXX2: 
       case DVIINST_XXX3: case DVIINST_XXX4:
-	len = (long)READ_INTN(PD(dvi,dvi_file), instr-DVIINST_XXX1+1);
+	len = (long)READ_UINTN(PD(dvi,dvi_file), instr-DVIINST_XXX1+1);
+#if 0
+	printf("Special: %d, %ld\n", instr, len);
+#endif
 	if (len > 0)
 	  special(dvi, dev, len);
 	break;
@@ -1345,7 +1433,10 @@ dvi_page(DVI dvi, DVI_DEVICE dev, int page, double shrink)
   }
 
 end_page:
+
+#if 1
   dvi_color_stack_save_page(dvi, dev, page);
+#endif
 
   return v;
 }
@@ -1375,7 +1466,7 @@ dvi_paper_size_in_dvi(DVI dvi, DVI_DEVICE dev)
 	break;
       if (cmd != NULL)
 	free(cmd); 
-      if ((cmd = malloc(len)) == NULL){
+      if ((cmd = malloc(len+1)) == NULL){
 	fprintf(stderr, "No memory\n");
 	exit(1);
       }
@@ -1569,7 +1660,7 @@ set_char(DVI dvi, DVI_DEVICE dev, UINT4 code_point, int instr,
 		     (long)(H_UNIT2PIXEL(HH)+off_x), 
 		     (long)(V_UNIT2PIXEL(VV)-off_y));
     } else {
-      dvi_put_bitmap(dev, dvi, &bm, -1, -1, -1,
+      dvi_put_bitmap(dev, dvi, &bm, -1, -1, (long)code_point,
 		     (long)(H_UNIT2PIXEL(HH)+off_x),
 		     (long)(V_UNIT2PIXEL(VV)-off_y));
     }
@@ -1672,21 +1763,27 @@ special(DVI dvi, DVI_DEVICE dev, long size)
 {
   char  *cmd     = NULL;
   int    i, v;
+  long   pos;
 
   if ((cmd = malloc(size+1)) == NULL){
     dvi->error = DVI_ERR_NO_MEMORY;
     DEV_CALL(dev,message_fatal)(dev, dvi, "No memory.");
     for (i = 0; i < size; i++)
       (void) READ_UINT1(PD(dvi,dvi_file));
-    free(cmd);
     return 0;
   }
   
+  pos = ftell(PD(dvi,dvi_file));
+  
   for (i = 0; i < size; i++)
     cmd[i] = READ_UINT1(PD(dvi,dvi_file));
   cmd[i] = '\0';
-
+#if 0
+  printf("%s\n", cmd);
+#endif
+  PD(dvi,special_pos) = pos;
   v = dvi_do_special(dvi, dev, cmd, H_UNIT2PIXEL(HH), V_UNIT2PIXEL(VV)); 
+  PD(dvi,special_pos) = -1;
 
   free(cmd);
   return v;
@@ -1780,9 +1877,9 @@ font_define(DVI dvi, DVI_DEVICE dev, UINT4 fnt, UINT4 check_sum,
 Private int
 font_do_open(DVI dvi, DVI_DEVICE dev, FONT f)
 {
-  char   fname[1024];
+  char   fname[MAXPATHLEN];
 
-  sprintf(fname, "%s.%s", f->fname, FONT_SUFFIX);
+  snprintf(fname, sizeof(fname), "%s.%s", f->fname, FONT_SUFFIX);
   
   f->fid = VF_OpenFont1(fname, dev->h_dpi, dev->v_dpi, -1, f->mag, f->mag);
   if (dvi_debug('O')){
@@ -1893,7 +1990,7 @@ font_get_metric(DVI dvi, DVI_DEVICE dev, UINT4 code_point,
 
   if (dvi_debug('M')){
     printf("DVIlib: VF_GetMetric1(%d,%ld,%p,%.3f,%.3f)\n",
-	   PD(dvi,current_fid), (long)code_point, &met,mx,my);
+	   PD(dvi,current_fid), (long)code_point, (void*)&met, mx, my);
   }
 
   if (VF_GetMetric1(PD(dvi,current_fid), 
@@ -1948,8 +2045,8 @@ font_load_bitmap(CACHE cache, CACHE_KEY key, int len)
   bmr = VF_RotatedBitmap(bm, VF_BM_ROTATE_270);
   if (dvi_debug('B')){
     printf("DVIlib: VF_RotatedBitmap(%p, %d);\n",
-	   bm, VF_BM_ROTATE_270);
-    printf("DVIlib:   ==> %p\n", bmr);
+	   (void*)bm, VF_BM_ROTATE_270);
+    printf("DVIlib:   ==> %p\n", (void*)bmr);
   }
 
   VF_FreeBitmap(bm);
@@ -1968,14 +2065,17 @@ altm_draw_ps_figures(DVI_DEVICE dev, DVI dvi)
   return 1;  /* Yes, please draw PS figures */
 }
 
-Private void
-altm_before_ps_figure(DVI_DEVICE dev, DVI dvi, char *spcmd, long x, long y)
+Private int
+altm_before_ps_figure(DVI_DEVICE dev, DVI dvi, char *spcmd, 
+		      long x, long y, long dvipos)
 {
-  return; /* Do nothing before PS figure */
+  /* Do nothing before PS figure */
+  return 1;   /* 1: draw PS fig / 0: skip PS fig */
 }
 
 Private void 
-altm_after_ps_figure(DVI_DEVICE dev, DVI dvi)
+altm_after_ps_figure(DVI_DEVICE dev, DVI dvi, char *spcmd, 
+		     long x, long y, long dvipos)
 {
   return; /* Do nothing after PS figure */
 }
