@@ -15,8 +15,12 @@ cat $RGB \
          printf("  { %-20s,\t0x%02x, 0x%02x, 0x%02x },\n",
                 "\"" tolower($4) " " tolower($5) "\"", $1, $2, $3); 
        }
-     }'  > __tmp1.rgb
-sort __tmp1.rgb > __tmp2.rgb
+     }' \
+ | sed 's/ /\$/g' \
+ > __tmp1.rgb
+sort __tmp1.rgb \
+ | sed 's/\$/ /g' \
+  > __tmp2.rgb
 
 #cat __tmp2.rgb
 #exit
