@@ -39,7 +39,7 @@
 #endif
 #include <ctype.h>
 
-#include "dvi-2_6.h"
+#include "libdvi29.h"
 #include "defs.h"
 #include "cache.h"
 #include "private.h"
@@ -54,6 +54,7 @@ static DVI_COLOR   dvi_color_stack_copy2(DVI,DVI_DEVICE,DVI_COLOR);
 static int         dvi_color_stack_equal(DVI_COLOR s1, DVI_COLOR s2);
 #endif
 
+static int   dvi_color_set(DVI,DVI_DEVICE,char*);
 static int   dvi_color_push(DVI,DVI_DEVICE,char*);
 static int   dvi_color_pop(DVI,DVI_DEVICE);
 
@@ -92,11 +93,13 @@ dvi_special_color(DVI dvi, DVI_DEVICE dev, char* cmd, long x, long y)
     p = &p[4];
     while (isspace((int)*p))
       p++;
+    if (*p == '\0')
+      p = NULL;
     res = dvi_color_push(dvi, dev, p);
   } else if (strncmp(p, "pop", 3) == 0){
     res = dvi_color_pop(dvi, dev);
   } else {
-    ; /* unknown command */
+    res = dvi_color_set(dvi, dev, p);
   }
 
 #if 0
@@ -106,6 +109,75 @@ dvi_special_color(DVI dvi, DVI_DEVICE dev, char* cmd, long x, long y)
   return res;
 }
 
+Public int  
+dvi_special_textcolor(DVI dvi, DVI_DEVICE dev, char* cmd, long x, long y)
+{
+  int   res;
+  char *p;
+
+  p = &cmd[strlen("textcolor:")];
+  while (isspace((int)*p))
+    p++;
+  if (*p == '\0')
+    return 0;
+
+  if (PD(dvi,color_stack) == NULL){
+    res = dvi_color_push(dvi, dev, p);
+  } else {
+    res = dvi_color_set(dvi, dev, p);
+  }
+
+  return res;
+}
+
+Public int  
+dvi_special_color_pdftex(DVI dvi, DVI_DEVICE dev, char* cmd, long x, long y)
+{
+  int    res;
+  char  *p;
+
+  p = &cmd[strlen("pdf: /C")];
+  while (isspace((int)*p))
+    p++;
+  if (*p == '\0')
+    return 0;
+
+  if (strncmp(p, ">>", 2) == 0){
+    res = dvi_color_pop(dvi, dev);
+  } else {
+    res = dvi_color_push(dvi, dev, p);
+  }
+
+  return 0;
+}
+
+
+
+static int
+dvi_color_set(DVI dvi, DVI_DEVICE dev, char *color_name)
+{
+  DVI_COLOR col;
+
+  if ((col = PD(dvi,color_stack)) == NULL)
+    return 0;
+
+  if (col->color_name != NULL){
+    free(col->color_name);
+    col->color_name = NULL;
+  }
+  if (color_name != NULL){
+    if ((col->color_name = (char*)malloc(strlen(color_name)+1)) == NULL)
+      return 0;
+    strcpy(col->color_name, color_name);
+    if (color_parse(dev, col, col->color_name) < 0){
+      free(col->color_name);
+      col->color_name = NULL;
+    }
+  }
+
+  return color_set(dvi, dev, col, color_name, COLOR_WHERE_FOREGROUND);
+}
+
 static int
 dvi_color_push(DVI dvi, DVI_DEVICE dev, char *color_name)
 {
@@ -115,7 +187,16 @@ dvi_color_push(DVI dvi, DVI_DEVICE dev, char *color_name)
     return -1;
   col->next = PD(dvi,color_stack);
   PD(dvi,color_stack) = col;
-  return color_set(dvi, dev, col, color_name, COLOR_WHERE_FOREGROUND);
+
+  if (color_name == NULL)
+    return 0;
+
+  while (isspace((int)*color_name))
+    color_name++;
+  if (*color_name == '\0')
+    return 0;
+
+  return dvi_color_set(dvi, dev, color_name);
 }
 
 static int
@@ -151,16 +232,16 @@ dvi_color_pop(DVI dvi, DVI_DEVICE dev)
 }
 
 
-Public int  
-dvi_special_bg_color(DVI dvi, DVI_DEVICE dev, char* cmd, long x, long y)
+
+
+static int  
+page_color(DVI dvi, DVI_DEVICE dev, char* color_name, long x, long y)
 {
   int        res;
   DVI_COLOR  col;
-  char       *color_name;
 
-  color_name = &cmd[strlen("background")];
-  while (isspace((int)*color_name))
-    color_name++;
+  if (color_name == NULL)
+    return 0;
   if (*color_name == '\0')
     return 0;
 
@@ -177,6 +258,38 @@ dvi_special_bg_color(DVI dvi, DVI_DEVICE dev, char* cmd, long x, long y)
 }
 
 
+Public int  
+dvi_special_bgcolor(DVI dvi, DVI_DEVICE dev, char* cmd, long x, long y)
+{
+  char       *color_name;
+
+  color_name = &cmd[strlen("background")];
+  while (isspace((int)*color_name))
+    color_name++;
+  if (*color_name == '\0')
+    return 0;
+
+  return  page_color(dvi, dev, color_name, x, y);
+}
+
+Public int  
+dvi_special_bgcolor_pdftex(DVI dvi, DVI_DEVICE dev, char* cmd, long x, long y)
+{
+  char  *color_name;
+
+  color_name = &cmd[strlen("pdf: /BG")];
+  while (isspace((int)*color_name))
+    color_name++;
+  if (*color_name == '\0')
+    return 0;
+
+  return  page_color(dvi, dev, color_name, x, y);
+
+  return 0;
+}
+
+
+
 
 
 
@@ -499,6 +612,7 @@ dvi_color_stack_elem_alloc(DVI dvi, DVI_DEVICE dev, char *color_name)
   col->c4         = -1;
   col->cname      = NULL;
   col->color_name = NULL;
+
   if (color_name != NULL){
     if ((col->color_name = (char*)malloc(strlen(color_name)+1)) == NULL){
       free(col);
@@ -569,6 +683,7 @@ color_parse(DVI_DEVICE dev, DVI_COLOR col, char *color_name)
 {
   char          *p;
   unsigned char  r, g, b;
+  int            ir, ig, ib; 
 
   if (color_name == NULL){
     color_name = COLOR_DEFAULT_RGB;
@@ -599,6 +714,11 @@ color_parse(DVI_DEVICE dev, DVI_COLOR col, char *color_name)
     col->model = COLOR_MODEL_GRAY;
     p = &p[4];
     sscanf(p, "%lf", &col->c1);
+  } else if (isdigit((int)*p) && (sscanf(p, "%i%i%i", &ir, &ig, &ib) == 3)){
+    col->model = COLOR_MODEL_RGB;
+    col->c1 = (double)ir/255.0;
+    col->c2 = (double)ig/255.0;
+    col->c3 = (double)ib/255.0;
   } else {
     if (DVI_rgb_lookup(color_name, &r, &g, &b) >= 0){
       col->model = COLOR_MODEL_RGB;
@@ -861,7 +981,7 @@ print_color(DVI_COLOR col)
     printf("NAMED %s", col->cname);
     break;
   default:
-    printf("??? unknown color model ???");
+    printf("???");
     break;
   }
 }
