@@ -6,7 +6,7 @@
  *
  */
 /*
- * Copyright (C) 1996, 1997 Hirotsugu Kakugawa. 
+ * Copyright (C) 1996-2001  Hirotsugu Kakugawa. 
  * All rights reserved.
  *
  * This file is part of the DVIlib Library.  This library is free
@@ -23,7 +23,10 @@
  */
 
 Public int  dvi_special_color(DVI,DVI_DEVICE,char*,long,long);
-Public int  dvi_special_bg_color(DVI,DVI_DEVICE,char*,long,long);
+Public int  dvi_special_bgcolor(DVI,DVI_DEVICE,char*,long,long);
+Public int  dvi_special_textcolor(DVI,DVI_DEVICE,char*,long,long);
+Public int  dvi_special_color_pdftex(DVI,DVI_DEVICE,char*,long,long);
+Public int  dvi_special_bgcolor_pdftex(DVI,DVI_DEVICE,char*,long,long);
 
 /* Default color: decision of acutual color is left to device drivers. */
 #define COLOR_DEFAULT_RGB   "rgb -1 -1 -1"
