@@ -18,7 +18,7 @@
  *
  */
 /*
- * Copyright (C) 1996-2000  Hirotsugu Kakugawa. 
+ * Copyright (C) 1996-2001  Hirotsugu Kakugawa. 
  * All rights reserved.
  *
  * This file is part of the DVIlib Library.  This library is free
@@ -50,15 +50,16 @@
 #  include  <strings.h>
 #endif
 #include <ctype.h>
+#include <signal.h>
+#include <limits.h>
 #include <sys/errno.h>
 #include <sys/param.h>
-#include <signal.h>
 #ifdef HAVE_FCNTL_H
 #  include <fcntl.h>
 #endif
 #if TIME_WITH_SYS_TIME
-# include <sys/time.h>
 # include <time.h>
+# include <sys/time.h>
 #else
 # if HAVE_SYS_TIME_H
 #  include <sys/time.h>
@@ -81,7 +82,7 @@
 
 extern int     errno;
 
-#include "dvi-2_6.h"
+#include "libdvi29.h"
 #include "defs.h"
 #include "cache.h"
 #include "spc_ps.h"
@@ -96,6 +97,7 @@ struct ps_str_arg {
   char  *name;
   int    namelen;
   char  *val;
+  int    vallen;
   int   *given;
 };
 struct  ps_float_arg {
@@ -132,13 +134,13 @@ Private void   put_frame(DVI_DEVICE,DVI,long,long,long,long);
 Private int    parse_special_postscriptbox(DVI,DVI_DEVICE,char*);
 Private double numberstr2pt(DVI,DVI_DEVICE,double,char*);
 Private int    parse_special_args(DVI,DVI_DEVICE,char*);
-static void    init_ps_info(struct ps *psi);
+Private void    init_ps_info(struct ps *psi);
 Private void   put_eps_figure(DVI,DVI_DEVICE,long,long);
 Private void   put_eps_bm2gray(DVI,DVI_DEVICE,long,long,DVI_BITMAP);
 Private void   put_eps_bm2rgb(DVI,DVI_DEVICE,long,long,DVI_BITMAP);
 Private void   put_eps_gray2rgb(DVI,DVI_DEVICE,long,long,DVI_GRAYMAP);
 Private int    eps2bitmap(DVI,DVI_DEVICE,int);
-Private FILE  *eps2psfile(DVI,DVI_DEVICE,char*,int);
+Private FILE  *eps2psfile(DVI,DVI_DEVICE,char*,int,int);
 Private int    eps_output_format(DVI_DEVICE dev);
 
 Private int    gs_convert_to_bitmap(DVI,DVI_DEVICE,char*,int,long,long);
@@ -160,10 +162,11 @@ Private void   gs_kill(void);
 #define  PSFIG_MAGIC_NUMBER  65781.76
 
 Public int 
-dvi_special_ps(DVI dvi, DVI_DEVICE dev, char *cmd, long x, long y)
+dvi_special_ps(DVI dvi, DVI_DEVICE dev, char *cmd, 
+	       long x, long y)
 {
   int     size_type, n;
-  char    s[1024];
+  char    s[4096];
   int     type, angle, push_angle;
   double  tmp, llx, lly, urx, ury;
   struct ps  *psi;
@@ -206,8 +209,8 @@ dvi_special_ps(DVI dvi, DVI_DEVICE dev, char *cmd, long x, long y)
       push_angle--;
     }
   } else if (strncmp(cmd, "ps:: ", 5) == 0){
-    char  s1[256], s2[256];
-    switch (sscanf(&cmd[5], "%s%s", s1, s2)){
+    char  s1[1024], s2[1024];
+    switch (sscanf(&cmd[5], "%1024s%1024s", s1, s2)){
     default:
       break; 
     case 1:    /* ps:: doclip ... unsupported. Figures are cliped. */
@@ -224,7 +227,7 @@ dvi_special_ps(DVI dvi, DVI_DEVICE dev, char *cmd, long x, long y)
       break;
     }
   } else if (strncmp(cmd, "ps: plotfile", 12) == 0){
-    n = sscanf(cmd, "%*s %*s %s", psi->param_filename);
+    n = sscanf(cmd, "%*s %*s %1024s", psi->param_filename);
     if (n == 1){
       psi->given_filename = 1;
       if ((psi->given_hsize == 1) && (psi->given_vsize == 1)){
@@ -282,7 +285,8 @@ dvi_special_ps(DVI dvi, DVI_DEVICE dev, char *cmd, long x, long y)
  **/
 
 Public int 
-dvi_special_postscriptbox(DVI dvi, DVI_DEVICE dev, char *cmd, long x, long y)
+dvi_special_postscriptbox(DVI dvi, DVI_DEVICE dev, char *cmd, 
+			  long x, long y)
 {
   struct ps  *psi;
   
@@ -358,7 +362,7 @@ parse_special_postscriptbox(DVI dvi, DVI_DEVICE dev, char *cmd)
     goto Error;
   if ((psi->param_vsize = numberstr2pt(dvi, dev, dev->v_dpi, param2)) < 0)
     goto Error;
-  strcpy(psi->param_filename, param3);
+  strncpy(psi->param_filename, param3, sizeof(psi->param_filename));
   psi->given_hsize = 1;
   psi->given_vsize = 1;
   psi->given_filename = 1;
@@ -406,7 +410,8 @@ Private int read_pgm(DVI,DVI_DEVICE,int,DVI_GRAYMAP,long,long);
 Private int read_bitmap_header(DVI,DVI_DEVICE,int,char*,long*,long*,int*);
 
 Public int 
-dvi_special_epsfile(DVI dvi, DVI_DEVICE dev, char *cmd, long x, long y)
+dvi_special_epsfile(DVI dvi, DVI_DEVICE dev, char *cmd, 
+		    long x, long y)
 {
   int         size_type;
   struct ps  *psi;
@@ -453,7 +458,8 @@ dvi_special_epsfile(DVI dvi, DVI_DEVICE dev, char *cmd, long x, long y)
  ** 'PSfile=' special
  **/
 Public int 
-dvi_special_PSfile(DVI dvi, DVI_DEVICE dev, char *cmd, long x, long y)
+dvi_special_PSfile(DVI dvi, DVI_DEVICE dev, char *cmd, 
+		   long x, long y)
 {
   struct ps  *psi;
   
@@ -514,7 +520,7 @@ parse_special_args(DVI dvi, DVI_DEVICE dev, char *cmd)
   int   i;
 
   for (i = 0; special_args_str[i].name != NULL; i++){
-    strcpy(special_args_str[i].val, ""); 
+    strncpy(special_args_str[i].val, "", special_args_str[i].vallen); 
     *special_args_str[i].given = 0;
   }
   for (i = 0; special_args_float[i].name != NULL; i++){
@@ -557,7 +563,7 @@ parse_special_args(DVI dvi, DVI_DEVICE dev, char *cmd)
       }
       ch = *p1;
       *p1 = '\0';
-      strcpy(special_args_str[i].val, p); 
+      strncpy(special_args_str[i].val, p, special_args_str[i].vallen); 
       *special_args_str[i].given = 1;
       *p1 = ch;
       p = p1;
@@ -616,20 +622,23 @@ init_ps_info(struct ps *psi)
   memset(psi, 0, sizeof(struct ps));
 
   i = 0;
-  special_args_str[i].name    = "epsfile=";
-  special_args_str[i].namelen = -1;
-  special_args_str[i].val     =  psi->param_filename;
-  special_args_str[i].given   = &psi->given_filename;
+  special_args_str[i].name     = "epsfile=";
+  special_args_str[i].namelen  = -1;
+  special_args_str[i].val      = psi->param_filename;
+  special_args_str[i].vallen   = sizeof(psi->param_filename);
+  special_args_str[i].given    = &psi->given_filename;
   i++;
-  special_args_str[i].name    = "PSfile=";
-  special_args_str[i].namelen = -1;
-  special_args_str[i].val     =  psi->param_filename;
-  special_args_str[i].given   = &psi->given_filename;
+  special_args_str[i].name     = "PSfile=";
+  special_args_str[i].namelen  = -1;
+  special_args_str[i].val      = psi->param_filename;
+  special_args_str[i].vallen   = sizeof(psi->param_filename);
+  special_args_str[i].given    = &psi->given_filename;
   i++;
-  special_args_str[i].name    = NULL;
-  special_args_str[i].namelen = -1;
-  special_args_str[i].val     = NULL;
-  special_args_str[i].given   = NULL;
+  special_args_str[i].name     = NULL;
+  special_args_str[i].namelen  = -1;
+  special_args_str[i].val      = NULL;
+  special_args_str[i].vallen   = 0;
+  special_args_str[i].given    = NULL;
 
   i = 0;
   special_args_float[i].name    = "hscale=";
@@ -722,7 +731,7 @@ eps2bitmap(DVI dvi, DVI_DEVICE dev, int size_type)
 {
   static int  last_format = EPS_OUTPUT_FORMAT_UNKNOWN;
   struct ps  *psi;
-  char    tempfile[1024];
+  char    tempfile[4096];
   FILE   *fp_psfile;
   int     max_level;
   long    w, h;
@@ -736,7 +745,8 @@ eps2bitmap(DVI dvi, DVI_DEVICE dev, int size_type)
     return -1;
   }
 
-  if ((fp_psfile = eps2psfile(dvi, dev, tempfile, size_type)) == NULL){
+  if ((fp_psfile = eps2psfile(dvi, dev, 
+			      tempfile, sizeof(tempfile), size_type)) == NULL){
     gs_invocation_nerr++;
     return -1;
   }
@@ -841,7 +851,8 @@ eps_output_format(DVI_DEVICE dev)
 
 
 Private FILE*
-eps2psfile(DVI dvi, DVI_DEVICE dev, char *tempfile, int size_type)
+eps2psfile(DVI dvi, DVI_DEVICE dev, 
+	   char *tempfile, int tempfile_size, int size_type)
 {
   struct ps *psi;
   char    *dvi_path, ps_path[MAXPATHLEN], buff[BUFSIZ], *p;
@@ -851,10 +862,10 @@ eps2psfile(DVI dvi, DVI_DEVICE dev, char *tempfile, int size_type)
   double   w, h, mx, my, w_tmp, h_tmp, r;
   int      r_tmp, r_angle;
 
-
   psi = &PD(dvi, ps_info);
 
-  sprintf(tempfile, "%s/dvilib.XXXXXX", DEV_CALL(dev,temp_dir)(dev, dvi));
+  snprintf(tempfile, tempfile_size, 
+	   "%s/dvilib.XXXXXX", DEV_CALL(dev,temp_dir)(dev, dvi));
 
 #ifdef HAVE_MKSTEMP
   if ((fd_eps_w = mkstemp(tempfile)) < 0)
@@ -877,13 +888,13 @@ eps2psfile(DVI dvi, DVI_DEVICE dev, char *tempfile, int size_type)
 
   dvi_path = PD(dvi, file_name);
   if (strrchr(dvi_path, '/') == NULL){
-    strcpy(ps_path, psi->param_filename);
+    strncpy(ps_path, psi->param_filename, sizeof(ps_path));
   } else {
-    strcpy(ps_path, dvi_path);
+    strncpy(ps_path, dvi_path, sizeof(ps_path));
     p = strrchr(ps_path, '/');
     *p = '\0';
-    strcat(ps_path, "/");
-    strcat(ps_path, psi->param_filename);
+    strncat(ps_path, "/", sizeof(ps_path));
+    strncat(ps_path, psi->param_filename, sizeof(ps_path));
   }
   if ((fp_in = fopen(ps_path, FOPEN_RD_MODE_TEXT)) == NULL){
     if ((fp_in = fopen(psi->param_filename, FOPEN_RD_MODE_TEXT)) == NULL){
@@ -1101,12 +1112,12 @@ eps2psfile(DVI dvi, DVI_DEVICE dev, char *tempfile, int size_type)
   ignore = 0;
   for (;;){
     int n;
-    char  p1[1024], p2[1024];
+    char  p1[4096], p2[4096];
 
     if (fgets(buff, sizeof(buff), fp_in) == NULL)
       break;
 
-    n = sscanf(buff, "%s %s", p1, p2);
+    n = sscanf(buff, "%4096s%4096s", p1, p2);
 
     /* ignore size specs */
     if ((n == 2) && (strcmp(p1, "%%PaperSize:") == 0)) {
@@ -1390,7 +1401,8 @@ put_eps_figure_bitmap(DVI dvi, DVI_DEVICE dev, DVI_BITMAP bm, long x, long y)
       step = EPS_BITMAP_INCREMENTAL_DISPLAY;
   }
   if (step < 1){
-    DEV_CALL(dev, put_bitmap)(dev, dvi, bm, -1, -1, -1, x, y);
+    DEV_CALL(dev, put_bitmap)(dev, dvi, bm, -1, 
+			      PD(dvi,special_pos), -1, x, y);
   } else {
     b0.width  = bm->width;
     b0.raster = bm->raster;
@@ -1399,7 +1411,8 @@ put_eps_figure_bitmap(DVI dvi, DVI_DEVICE dev, DVI_BITMAP bm, long x, long y)
       b0.height = step;
       if (i + step > bm->height)
 	b0.height = bm->height - i;
-      DEV_CALL(dev, put_bitmap)(dev, dvi, &b0, -1, -1, -1, x, y+i);
+      DEV_CALL(dev, put_bitmap)(dev, dvi, &b0, -1, 
+				PD(dvi,special_pos), -1, x, y+i);
     }
   }
 }
@@ -1422,7 +1435,8 @@ put_eps_figure_graymap(DVI dvi, DVI_DEVICE dev, DVI_GRAYMAP gm, long x, long y)
       step = EPS_BITMAP_INCREMENTAL_DISPLAY;
   }
   if (step < 1){
-    DEV_CALL(dev, put_graymap)(dev, dvi, gm, -1, -1, -1, x, y);
+    DEV_CALL(dev, put_graymap)(dev, dvi, gm, -1, 
+			       PD(dvi,special_pos), -1, x, y);
   } else {
     g0.width  = gm->width;
     g0.raster = gm->raster;
@@ -1432,7 +1446,8 @@ put_eps_figure_graymap(DVI dvi, DVI_DEVICE dev, DVI_GRAYMAP gm, long x, long y)
       g0.height = step;
       if (i + step > gm->height)
 	g0.height = gm->height - i;
-      DEV_CALL(dev, put_graymap)(dev, dvi, &g0, -1, -1, -1, x, y+i);
+      DEV_CALL(dev, put_graymap)(dev, dvi, &g0, -1, 
+				 PD(dvi,special_pos), -1, x, y+i);
     }
   }
 }
@@ -1457,7 +1472,8 @@ put_eps_figure_pixmap_rgb(DVI dvi, DVI_DEVICE dev, DVI_PIXMAP_RGB pm,
       step = EPS_BITMAP_INCREMENTAL_DISPLAY;
   }
   if (step < 1){
-    DEV_CALL(dev, put_pixmap_rgb)(dev, dvi, pm, -1, -1, -1, x, y);
+    DEV_CALL(dev, put_pixmap_rgb)(dev, dvi, pm, -1, 
+				  PD(dvi,special_pos), -1, x, y);
   } else {
     p0.width  = pm->width;
     p0.raster = pm->raster;
@@ -1467,7 +1483,8 @@ put_eps_figure_pixmap_rgb(DVI dvi, DVI_DEVICE dev, DVI_PIXMAP_RGB pm,
       p0.height = step;
       if (i + step > pm->height)
 	p0.height = pm->height - i;
-      DEV_CALL(dev, put_pixmap_rgb)(dev, dvi, &p0, -1, -1, -1, x, y+i);
+      DEV_CALL(dev, put_pixmap_rgb)(dev, dvi, &p0, -1, 
+				    PD(dvi,special_pos), -1, x, y+i);
     }
   }
 }
@@ -1701,22 +1718,26 @@ Private int
 gs_convert_to_bitmap(DVI dvi, DVI_DEVICE dev, char *psfile, 
 		     int bm_format, long bm_width, long bm_height)
 {
-  char  buff[1024];
+  char  buff[4096];
 
   if (start_ghostscript_process(dvi, dev, bm_format, 0) < 0)
     return -1;
 
+
   DEV_CALL(dev,message_advice)(dev, dvi, "Drawing a PostScript figure...");
 
-  sprintf(buff, "[ /HWSize [ %ld %ld ] currentdevice putdeviceprops pop\n",
+  snprintf(buff, sizeof(buff),
+	   "[ /HWSize [ %ld %ld ] currentdevice putdeviceprops pop\n",
 	  bm_width, bm_height);
   if (getenv("DVILIB_DEBUG_EPS") != NULL){
     printf("Send to Ghostscrip: %s", buff);
   }
+
   if (send_to_gs(buff) < 0)
     return -1;
 
-  sprintf(buff, "(%s) run flush\n", psfile);
+  snprintf(buff, sizeof(buff),
+	   "(%s) run flush\n", psfile);
   if (getenv("DVILIB_DEBUG_EPS") != NULL){
     printf("Send to Ghostscrip: %s", buff);
   }
@@ -1809,14 +1830,16 @@ start_ghostscript_process(DVI dvi, DVI_DEVICE dev, int bm_format, int first)
   return 0;
 }
 
+
 Private int
 try_start_ghostscript_process(DVI dvi, DVI_DEVICE dev, 
 			      int bm_format, int first)
 {
   int   i;
   int   pipe_dvi2gs[2], pipe_gs2dvi[2];
-  char  *gs_output_device, *gs_argv[16];
-  char  OptFormat[128];
+  char  *gs_output_device, *gs_argv[32];
+  char  OptFormat[1024];
+  static int  hander_installed = 0;
 
   if (first == 1)
     DEV_CALL(dev,message_advice)(dev, dvi, "Invoking GhostScript...");
@@ -1838,7 +1861,7 @@ try_start_ghostscript_process(DVI dvi, DVI_DEVICE dev,
   gs_argv[i++] = DEV_CALL(dev,gs_program_path)(dev, dvi);
   gs_argv[i++] = "-dQUIET";
   gs_argv[i++] = "-dNOPAUSE";
-  sprintf(OptFormat, "-sDEVICE=%s", gs_output_device);
+  snprintf(OptFormat, sizeof(OptFormat), "-sDEVICE=%s", gs_output_device);
   gs_argv[i++] = OptFormat;
   gs_argv[i++] = "-sOutputFile=-"; 
   gs_argv[i++] = "-";
@@ -1863,14 +1886,15 @@ try_start_ghostscript_process(DVI dvi, DVI_DEVICE dev,
     return -1;
   }
 
+  if (hander_installed == 0){
 #ifdef SIGCHLD
-#if 1
-  signal(SIGCHLD,  gs_when_died);
-#endif
+    signal(SIGCHLD,  gs_when_died);
 #endif
 #ifdef SIGPIPE
-  signal(SIGPIPE,  gs_when_died);
+    signal(SIGPIPE,  gs_when_died);
 #endif
+    hander_installed = 1;
+  }
 
   GS_Pipe_To_GS    = -1;
   GS_Pipe_From_GS  = -1;
@@ -1896,6 +1920,19 @@ try_start_ghostscript_process(DVI dvi, DVI_DEVICE dev,
     dup2(pipe_gs2dvi[1], 1);
     close(pipe_gs2dvi[0]);
     close(pipe_gs2dvi[1]);
+    /* close other fds */
+    { 
+      int fd;
+      static int om = 0;
+#ifdef _SC_OPEN_MAX
+      if (om == 0)
+	om = sysconf(_SC_OPEN_MAX);
+#else
+      om = 1024;
+#endif
+      for (fd = 3; fd < om; fd++)
+	(void)close(fd);
+    }
     /* exec gs */
     execvp(DEV_CALL(dev,gs_program_path)(dev, dvi), gs_argv);
     if (gs_invocation_nerr == 0)
@@ -1934,21 +1971,27 @@ gs_when_died(int sig)
 {
   int  pid, pst;
 
+#if 0
+  printf("SIGNAL %d\n", sig);
+#endif
+
   if (GS_ProcessID < 0)
     return;
 
   do {
-#ifdef HAVE_WAITPID
+#if defined(HAVE_WAITPID)
     /* we use WNOHANG option, since this routine may be called 
      * by SIGCHLD of a process other than ghostscript */
     pid = waitpid(GS_ProcessID, &pst, WNOHANG);
-    if (pid == 0)      /* SIGCHLD by a non-gs process */
-      return;
+#elif defined(HAVE_WAIT4)
+    pid = wait4(GS_ProcessID, &pst, WNOHANG, NULL);
+#elif defined(HAVE_WAIT2)
+    pid = wait3(GS_ProcessID, WNOHANG, NULL);
 #else
     /* XXX wrong code (incorrect when non-gs process terminates) */
     pid = wait(&pst);
-    if (pid != GS_ProcessID)
-      return;
+    if (pid == GS_ProcessID)
+      break;
 #endif
   } while ((pid == -1) && (errno == EINTR));
   
@@ -1961,17 +2004,21 @@ gs_when_died(int sig)
   
   GS_ProcessStatus = GS_STAT_DIED;
   GS_ProcessID     = -1;
-  if (GS_Pipe_To_GS >= 0)
+  if (GS_Pipe_To_GS >= 0){
     close(GS_Pipe_To_GS);
-  GS_Pipe_To_GS = -1;
-  if (GS_Pipe_From_GS >= 0)
+    GS_Pipe_To_GS = -1;
+  }
+  if (GS_Pipe_From_GS >= 0){
     close(GS_Pipe_From_GS);
-  GS_Pipe_From_GS = -1;
+    GS_Pipe_From_GS = -1;
+  }
 }
 
 Private void
 gs_kill(void)
 {
+  printf("gs_kill\n");
+
   gs_invocation_nerr = 0;
 
   if ((GS_ProcessStatus == GS_STAT_RUNNING)
@@ -1984,7 +2031,7 @@ gs_kill(void)
       close(GS_Pipe_From_GS);
     GS_Pipe_From_GS = -1;
 
-#if 1
+#if 0
     GS_ProcessStatus = GS_STAT_DIED;
     GS_ProcessID     = -1;
 #else
@@ -2016,12 +2063,13 @@ gs_convert_to_bitmap(DVI dvi, DVI_DEVICE dev,
 {
   int    pst;
   char  *gs_dev;
-  char   OptFormat[128], OptBitmapSize[256], OptOutput[MAXPATHLEN+16];
+  char   OptFormat[128], OptBitmapSize[256], OptOutput[MAXPATHLEN];
 
   GS_OutputFileFD = -1;
 
-  sprintf(GS_OutputFileName, "%s/dvi-XXXXXX.bm",
-	  DEV_CALL(dev,temp_dir)(dev,dvi));
+  snprintf(GS_OutputFileName, sizeof(GS_OutputFileName),
+	   "%s/dvi-XXXXXX.bm",
+	   DEV_CALL(dev,temp_dir)(dev,dvi));
   tmpnam(GS_OutputFileName);
   if (strcmp(GS_OutputFileName, "") == 0)
     return -1;
@@ -2039,9 +2087,12 @@ gs_convert_to_bitmap(DVI dvi, DVI_DEVICE dev,
     break;
   }
 
-  sprintf(OptFormat,     "-sDEVICE=%s", gs_dev);
-  sprintf(OptBitmapSize, "-g%ldx%ld", bm_width, bm_height);
-  sprintf(OptOutput,     "-sOutputFile=%s", GS_OutputFileName);
+  snprintf(OptFormat,     sizeof(OptFormat), 
+	   "-sDEVICE=%s", gs_dev);
+  snprintf(OptBitmapSize, sizeof(OptBitmapSize),
+	   "-g%ldx%ld", bm_width, bm_height);
+  snprintf(OptOutput,     sizeof(OptOutput),
+	   "-sOutputFile=%s", GS_OutputFileName);
 
 #ifdef HAVE_FORK
   {
@@ -2097,20 +2148,32 @@ gs_convert_to_bitmap(DVI dvi, DVI_DEVICE dev,
   {
     char  gs_cmd[3*MAXPATHLEN];
 
-    strcpy(gs_cmd, "");
-    strcat(gs_cmd, DEV_CALL(dev,gs_program_path)(dev, dvi));
-    strcat(gs_cmd, " -dQUIET");
-    strcat(gs_cmd, " -dNOPAUSE");
-    strcat(gs_cmd, " ");
-    strcat(gs_cmd, OptFormat);
-    strcat(gs_cmd, " ");
-    strcat(gs_cmd, OptBitmapSize);
-    strcat(gs_cmd, " ");
-    strcat(gs_cmd, OptOutput); 
-    strcat(gs_cmd, " ");
-    strcat(gs_cmd, psfile);
-    strcat(gs_cmd, " ");
-    strcat(gs_cmd, "quit.ps");
+    snprintf(gs_cmd, sizeof(gs_cmd), 
+	     "%s %s %s %s %s %s %s %s", 
+	     DEV_CALL(dev,gs_program_path)(dev, dvi), 
+	     "-dQUIET",
+	     "-dNOPAUSE", 
+	     OptFormat,
+	     OptBitmapSize, 
+	     OptOutput,
+	     psfile,
+	     "quit.ps");
+#if 0
+    strncpy(gs_cmd, "", sizeof(gs_cmd));
+    strncat(gs_cmd, DEV_CALL(dev,gs_program_path)(dev, dvi), sizeof(gs_cmd));
+    strncat(gs_cmd, " -dQUIET", sizeof(gs_cmd));
+    strncat(gs_cmd, " -dNOPAUSE", sizeof(gs_cmd));
+    strncat(gs_cmd, " ", sizeof(gs_cmd));
+    strncat(gs_cmd, OptFormat, sizeof(gs_cmd));
+    strncat(gs_cmd, " ", sizeof(gs_cmd));
+    strncat(gs_cmd, OptBitmapSize, sizeof(gs_cmd));
+    strncat(gs_cmd, " ", sizeof(gs_cmd));
+    strncat(gs_cmd, OptOutput, sizeof(gs_cmd)); 
+    strncat(gs_cmd, " ", sizeof(gs_cmd));
+    strncat(gs_cmd, psfile, sizeof(gs_cmd));
+    strncat(gs_cmd, " ", sizeof(gs_cmd));
+    strncat(gs_cmd, "quit.ps", sizeof(gs_cmd));
+#endif
 
     if (getenv("DVILIB_DEBUG_EPS") != NULL){
       int  j;
