@@ -46,7 +46,7 @@
 #if HAVE_SIGNAL_H
 #  include <signal.h>
 #endif
-#include "dvi-2_6.h"
+#include "libdvi29.h"
 #include "defs.h"
 #include "cf-dvi2escpage.h"
 #include "compiled.h"
