@@ -8,9 +8,12 @@
  *  19 Jan 1999   Added dvi2escpg (EPSON ESC/Page output) feature
  *                (snarfed from dvi2escpage.c) and dvi2rpdl (RICOH RPDL
  *                printer output, e.g., NX100, NX700) feature.
+ *  24 Jan 2002   Added pipe output for PPM and PGM output mode.
+ *                For example, "-ofile '| convert - foo.png'",
+ *  15 Apr 2005   Double side print for RPDL printers
  */
 /*
- * Copyright (C) 1998-1999  Hirotsugu Kakugawa. 
+ * Copyright (C) 1998-2007  Hirotsugu Kakugawa. 
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -49,7 +52,7 @@
 #  include <signal.h>
 #endif
 
-#include "dvi-2_6.h"
+#include "libdvi29.h"
 #include "defs.h"
 #include "cf-dvi2misc.h"
 #include "../params.h"
@@ -71,6 +74,7 @@ char         *param_vflibcap    = DEFAULT_VFLIBCAP;
 char         *param_mode        = NULL;
 int           param_dpi         = 0;
 int           param_landscape   = FALSE;
+int           param_doubleside  = FALSE;
 double        param_offset_x    = 0;
 double        param_offset_y    = 0;
 char         *param_device      = NULL;
@@ -100,14 +104,20 @@ long          offset_y    = 0;
 
 
 char *color_fg = "black";
-unsigned char  color_fg_r = 0;
-unsigned char  color_fg_g = 0;
-unsigned char  color_fg_b = 0;
+unsigned char  color_fg_r         = 0;
+unsigned char  color_fg_g         = 0;
+unsigned char  color_fg_b         = 0;
+unsigned char  color_fg_r_default = 0;
+unsigned char  color_fg_g_default = 0;
+unsigned char  color_fg_b_default = 0;
 
 char *color_bg = "white";
-unsigned char  color_bg_r = 0xff;
-unsigned char  color_bg_g = 0xff;
-unsigned char  color_bg_b = 0xff;
+unsigned char  color_bg_r         = 0xff;
+unsigned char  color_bg_g         = 0xff;
+unsigned char  color_bg_b         = 0xff;
+unsigned char  color_bg_r_default = 0xff;
+unsigned char  color_bg_g_default = 0xff;
+unsigned char  color_bg_b_default = 0xff;
 
 
 
@@ -301,6 +311,9 @@ parse_args(int argc, char **argv)
     } else if (strcmp(*argv, "-port") == 0){
       param_landscape = FALSE;
 
+    } else if (strcmp(*argv, "-ds") == 0){
+      param_doubleside = TRUE;
+
     } else if ((strcmp(*argv, "-ox") == 0) && (argc > 1)){
       argc--;  argv++;
       param_offset_x = atof(*argv);
@@ -325,10 +338,12 @@ parse_args(int argc, char **argv)
     } else if ((strcmp(*argv, "-from") == 0) && (argc > 1)){
       argc--;  argv++;
       param_page_from = atoi(*argv);
+      param_dvi2img_page  = -1;
 
     } else if ((strcmp(*argv, "-to") == 0) && (argc > 1)){
       argc--;  argv++;
       param_page_to = atoi(*argv);
+      param_dvi2img_page  = -1;
 
     } else if (strcmp(*argv, "-rev") == 0){
       param_page_rev = 1;
@@ -368,11 +383,23 @@ parse_args(int argc, char **argv)
 
     } else if ((strcmp(*argv, "-fg") == 0) && (argc > 1)){       /* ppm */
       argc--;  argv++;
-      DVI_rgb_lookup(*argv, &color_fg_r, &color_fg_g, &color_fg_b);
+      if (DVI_rgb_lookup(*argv, &color_fg_r, &color_fg_g, &color_fg_b) < 0){
+	fprintf(stderr, "Unknown foreground color: %s\n", *argv);
+	exit(1);
+      }
+      color_fg_r_default = color_fg_r;
+      color_fg_g_default = color_fg_g;
+      color_fg_b_default = color_fg_b;
 
     } else if ((strcmp(*argv, "-bg") == 0) && (argc > 1)){       /* ppm */
       argc--;  argv++;
-      DVI_rgb_lookup(*argv, &color_bg_r, &color_bg_g, &color_bg_b);
+      if (DVI_rgb_lookup(*argv, &color_bg_r, &color_bg_g, &color_bg_b) < 0){
+	fprintf(stderr, "Unknown background color: %s\n", *argv);
+	exit(1);
+      }
+      color_bg_r_default = color_bg_r;
+      color_bg_g_default = color_bg_g;
+      color_bg_b_default = color_bg_b;
 
     } else if (strcmp(*argv, "-300") == 0){                        /* escpg */
       param_dpi = 300;
@@ -474,7 +501,7 @@ clean_up(int exit_val)
 RETSIGTYPE
 sig_handler(int sig)
 {
-  fprintf(stderr, "%s: Interrupted.\n", PROGRAM);
+  fprintf(stderr, "Interrupted.\n");
   interrupted = 1;
 
   switch (param_program){
@@ -541,6 +568,7 @@ usage(void)
     PR0("  -600         600 dpi mode, same as \"-dpi 600 -mode ljfour\"\n");
   }
   if (param_program == PROGRAM_TYPE_DVI2RPDL){
+    PR0("  -ds          Double side print\n");
     PR0("  -n N         Number of copies [1]\n");
   }
   PR0("  -defaults    Print defaut values\n");
@@ -621,7 +649,7 @@ dvi2misc(char *dvi_file)
 
   switch (param_program){
   default:
-    abort();
+    usage();
     /* devices that accumulate glyphs in a frame buffer (mono/gray) */
   case PROGRAM_TYPE_DVI2BDPS:
   case PROGRAM_TYPE_DVI2PGM:
@@ -660,7 +688,6 @@ dvi2misc(char *dvi_file)
     clean_up(1);
   }
 
-
   if (param_page_from >= 1)
     param_page_from = find_page1(dvi, param_page_from);
   if (param_page_from <  1)
@@ -774,10 +801,20 @@ DEV_color_rgb(DVI_DEVICE dev, DVI dvi, int page_resume_flag,
 #if 0
   printf("FOREGROUND COLOR %.3f %.3f %.3f\n", r, g, b);
 #endif
-  color_fg_r = 255 * r;
-  color_fg_g = 255 * g; 
-  color_fg_b = 255 * b;
+  if ((r >= 0) && (g >= 0) && (b >= 0)){
+    color_fg_r = 255 * r;
+    color_fg_g = 255 * g; 
+    color_fg_b = 255 * b;
+  } else {
+    color_fg_r = color_fg_r_default;
+    color_fg_g = color_fg_g_default;
+    color_fg_b = color_fg_b_default;
+  }
   DVI_fb_rgb_foreground(framebuff, color_fg_r, color_fg_g, color_fg_b);
+#if 0
+    fprintf(stderr, "FG CHANGE: %.3f %.3f %.3f  => %02x %02x %02x\n", 
+	    r, g, b, color_fg_r, color_fg_g, color_fg_b);
+#endif
   return; 
 }
 
@@ -788,9 +825,20 @@ DEV_bgcolor_rgb(DVI_DEVICE dev, DVI dvi, int page_resume_flag,
 #if 0
   printf("BACKGROUND COLOR %.3f %.3f %.3f\n", r, g, b);
 #endif
-  color_bg_r = 255 * r;
-  color_bg_g = 255 * g; 
-  color_bg_b = 255 * b;
+  if ((r >= 0) && (g >= 0) && (b >= 0)){
+    color_bg_r = 255 * r;
+    color_bg_g = 255 * g; 
+    color_bg_b = 255 * b;
+  } else {
+    color_bg_r = color_bg_r_default;
+    color_bg_g = color_bg_g_default;
+    color_bg_b = color_bg_b_default;
+  }
+#if 0
+    fprintf(stderr, "BG CHANGE: %02x %02x %02x\n", 
+	    color_bg_r, color_bg_g, color_bg_b);
+#endif
+
   DVI_fb_rgb_background(framebuff, color_bg_r, color_bg_g, color_bg_b);
   DVI_fb_clear(framebuff);
   return;
@@ -820,4 +868,35 @@ DEV_message_ignore(DVI_DEVICE dev, DVI dvi, char *fmt,...)
 }
 
 
+FILE*
+output_file_fopen(char *ofile) 
+{
+  FILE  *fp = NULL;
+
+  if (ofile == NULL)
+    return NULL;
+
+  if (ofile[0] == '|'){   /* pipe */
+    fp = popen(&ofile[1], "w");
+  } else {
+    fp = fopen(ofile, "w");
+  }
+  return fp;
+}
+
+void
+output_file_fclose(char *ofile, FILE *fp) 
+{
+  if (fp == NULL)
+    return;
+  if (ofile == NULL)
+    return;
+
+  if (ofile[0] == '|'){   /* pipe */
+    pclose(fp);
+  } else {
+    fclose(fp);
+  }
+}
+
 /*EOF*/
