@@ -43,10 +43,12 @@
 #  include <strings.h>
 #endif
 #include <ctype.h>
-#include <dvi-2_6.h>
+
 #include <gtk/gtk.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
+
+#include "libdvi29.h"
 #include "defs.h"
 
 #define AA      4        /* must be integer value and be greater than 1 */
