@@ -16,7 +16,7 @@
 #include <X11/Xutil.h>
 #include <X11/cursorfont.h>
 #include <X11/keysym.h>
-#include "dvi-2_6.h"
+#include "libdvi29.h"
 #include "defs.h"
 #define SPAWX11_VERSION   "1.2.2" 
 
@@ -58,8 +58,8 @@ int   main_loop(DVI_DEVICE,DVI);
 DVI_DEVICE  create_dev(void);
 void  DEV_put_bitmap(DVI_DEVICE,DVI,DVI_BITMAP,int,long,long,long,long);
 void  DEV_put_rectangle(DVI_DEVICE,DVI,long,long,long,long);
-void  DEV_before_ps_figure(DVI_DEVICE,DVI,char*,long,long);
-void  DEV_after_ps_figure(DVI_DEVICE,DVI);
+int   DEV_before_ps_figure(DVI_DEVICE,DVI,char*,long,long,long);
+void  DEV_after_ps_figure(DVI_DEVICE,DVI,char*,long,long,long);
 int   DEV_draw_ps_figures(DVI_DEVICE,DVI);
 int   DEV_poll(DVI_DEVICE,DVI,int);
 void  x_init(void);
@@ -375,12 +375,15 @@ void  DEV_put_rectangle(DVI_DEVICE dev, DVI dvi,
 	   w+param_aa, h+param_aa);
 }
 
-void  DEV_before_ps_figure(DVI_DEVICE dev, DVI dvi, char *cmd, long x, long y)
+int   DEV_before_ps_figure(DVI_DEVICE dev, DVI dvi, char *cmd, 
+			   long x, long y, long dvipos)
 {
   x_change_cursor(x_c_ps);
+  return 1;
 }
 
-void  DEV_after_ps_figure(DVI_DEVICE dev, DVI dvi)
+void  DEV_after_ps_figure(DVI_DEVICE dev, DVI dvi, char *cmd, 
+			  long x, long y, long dvipos)
 {
   x_change_cursor(x_c_drawing);
 }
