@@ -33,8 +33,8 @@
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <dvi-2_6.h>
 
+#include "libdvi29.h"
 #include "defs.h"
 #include "xgdvi.h"
 #include "files.h"
@@ -152,13 +152,13 @@ create_Printer ()
     debug_lpr_command = 1;
 
   lprconf_read(XGDVI_CONFIG_DIR, _("PrintSpec16"));
-
+	       
   Printer = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
   gtk_container_set_border_width (GTK_CONTAINER (Printer), 10);
   gtk_window_set_title (GTK_WINDOW (Printer), _("xgdvi: Print Document"));
   gtk_object_set_data (GTK_OBJECT (xgdvi), "Printer", Printer);
-#if 0
+#if 1
   gtk_widget_set_events(Printer, GDK_KEY_PRESS_MASK);
   gtk_signal_connect(GTK_OBJECT(Printer), "key_press_event", 
 		     GTK_SIGNAL_FUNC(event_key_press), NULL);
@@ -242,7 +242,7 @@ create_Printer ()
   gtk_container_add (GTK_CONTAINER (vbox2), hbox22);
   gtk_widget_show (hbox22);
 
-  prButtonPage1 = gtk_button_new_with_label ("Print 1 Page");
+  prButtonPage1 = gtk_button_new_with_label (_("Print 1 Page"));
   gtk_widget_show (prButtonPage1);
   gtk_box_pack_start (GTK_BOX (hbox22), prButtonPage1, TRUE, TRUE, 0);
   gtk_container_border_width (GTK_CONTAINER (prButtonPage1), 5);
@@ -499,9 +499,11 @@ lpr_abort_command()
 
   lpr_running_command = 0;
   gtk_widget_unmap (prButtonAbort);
-  if (lpr_pipe_fp != NULL)
+  gdk_flush();
+  if (lpr_pipe_fp != NULL){
     pclose(lpr_pipe_fp);
-  lpr_pipe_fp = NULL;
+    lpr_pipe_fp = NULL;
+  }
   gdk_input_remove(lpr_handler_id);
   lpr_handler_id = -1;
   lpr_message(msg2, strlen(msg2), 1);
@@ -521,9 +523,10 @@ lpr_message_callback(gpointer data, gint fd, GdkInputCondition cond)
     gtk_widget_unmap (prButtonAbort);
     gdk_input_remove(lpr_handler_id);
     lpr_handler_id = -1;
-    if (lpr_pipe_fp != NULL)
+    if (lpr_pipe_fp != NULL){
       pclose(lpr_pipe_fp);
-    lpr_pipe_fp = NULL;
+      lpr_pipe_fp = NULL;
+    }
     return;
   }
 
@@ -553,6 +556,7 @@ lpr_run_command(char *spec, int act, char *job)
   if (dvi == NULL){
     p = 0;
     np = 0;
+    ap = 0; 
     rb = re = 0; 
   } else {
     p = page;
