@@ -33,10 +33,10 @@
 #  include <strings.h>
 #endif
 
-#include <dvi-2_6.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
+#include "libdvi29.h"
 #include "defs.h"
 #include "xgdvi.h"
 #include "gui.h"
@@ -76,11 +76,6 @@ on_PageList_clicked                    (GtkWidget *widget,
 					gpointer user_data);
 
 
-gint
-event_key_press(GtkWidget *w, GdkEventKey *ev);
-
-
-
 
 
 
@@ -95,9 +90,9 @@ create_xgdvi ()
   GtkWidget *License;
   GtkWidget *Exit;
   GtkWidget *Box;
-  GtkWidget *hbox0;
+  GtkWidget *CmdHBoxTop;
   GtkWidget *hbox1;
-  GtkWidget *vbox2;
+  GtkWidget *CmdVBoxLeft;
   GtkWidget *hsep;
   GtkWidget *Reload;
   GtkWidget *ZoomIn;
@@ -122,7 +117,7 @@ create_xgdvi ()
 
   xgdvi = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_object_set_data (GTK_OBJECT (xgdvi), "xgdvi", xgdvi);
-  gtk_window_set_title (GTK_WINDOW (xgdvi), _("xgdvi"));
+  gtk_window_set_title (GTK_WINDOW (xgdvi), param_win_title);
   gtk_window_set_policy (GTK_WINDOW (xgdvi), TRUE, TRUE, FALSE);
   gtk_widget_set_events(xgdvi, GDK_EXPOSURE_MASK|GDK_KEY_PRESS_MASK);
   gtk_signal_connect(GTK_OBJECT(xgdvi), "configure_event",
@@ -131,8 +126,6 @@ create_xgdvi ()
 		     GTK_SIGNAL_FUNC(event_key_press), NULL);
   gtk_signal_connect(GTK_OBJECT(xgdvi), "destroy", 
 		     GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
-  gtk_window_set_default_size(GTK_WINDOW(xgdvi), 
-			      param_win_size_w, param_win_size_h);
   gtk_window_set_policy(GTK_WINDOW(xgdvi), 1, 1, 1);
 
   vbox = gtk_vbox_new (FALSE, 3);
@@ -141,27 +134,29 @@ create_xgdvi ()
   gtk_container_add (GTK_CONTAINER (xgdvi), vbox);
 
   if ((param_gui_type == 1) || (param_gui_type == 2)){
-    hbox0 = NULL;
+    CmdHBoxTop = NULL;
   } else {
-    hbox0 = gtk_hbox_new(FALSE, 0);
-    gtk_widget_show (hbox0);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox0, FALSE, FALSE, 0);
+    CmdHBoxTop = gtk_hbox_new(FALSE, 0);
+    gtk_widget_show (CmdHBoxTop);
+    gtk_box_pack_start (GTK_BOX (vbox), CmdHBoxTop, FALSE, FALSE, 0);
   } 
 
   hbox1 = gtk_hbox_new(FALSE, 0);
   gtk_widget_show (hbox1);
   gtk_box_pack_start (GTK_BOX (vbox), hbox1, TRUE, TRUE, 0);
 
-  vbox2 = gtk_vbox_new (FALSE, 0);
-  gtk_widget_show (vbox2);
-  gtk_box_pack_start (GTK_BOX (hbox1), vbox2, FALSE, FALSE, 3);
+  CmdVBoxLeft = gtk_vbox_new (FALSE, 0);
+  gtk_widget_show (CmdVBoxLeft);
+  gtk_box_pack_start (GTK_BOX (hbox1), CmdVBoxLeft, FALSE, FALSE, 3);
 
   if ((param_gui_type == 1) || (param_gui_type == 2)){
-    Box = vbox2;
+    Box = CmdVBoxLeft;
   } else {
-    Box = hbox0;
+    Box = CmdHBoxTop;
   }
 
+  gtk_object_set_data (GTK_OBJECT (xgdvi), "CmdBoxTop", CmdHBoxTop);
+  gtk_object_set_data (GTK_OBJECT (xgdvi), "CmdBoxLeft", CmdVBoxLeft);
 
   Reload = gtk_button_new_with_label (_("Reload"));
   if (no_focus == 1)
@@ -318,14 +313,14 @@ create_xgdvi ()
   if ((param_gui_type == 1) || (param_gui_type == 2)){
     hsep = gtk_hseparator_new();
     gtk_widget_show (hsep);
-    gtk_box_pack_start (GTK_BOX (vbox2), hsep, FALSE, TRUE, 5);
+    gtk_box_pack_start (GTK_BOX (CmdVBoxLeft), hsep, FALSE, TRUE, 5);
   }
 
   FirstPage = gtk_button_new_with_label (_("First Page"));
   if (no_focus == 1)
     GTK_WIDGET_UNSET_FLAGS(FirstPage, GTK_CAN_FOCUS);
   gtk_widget_show (FirstPage);
-  gtk_box_pack_start (GTK_BOX (vbox2), FirstPage, FALSE, TRUE, 3);
+  gtk_box_pack_start (GTK_BOX (CmdVBoxLeft), FirstPage, FALSE, TRUE, 3);
   gtk_signal_connect (GTK_OBJECT (FirstPage), "clicked",
                       GTK_SIGNAL_FUNC (on_GotoPage_clicked), "<"); 
   gtk_tooltips_set_tip(gtk_tooltips_new(), FirstPage,
@@ -336,7 +331,7 @@ create_xgdvi ()
   if (no_focus == 1)
     GTK_WIDGET_UNSET_FLAGS(PrevPage, GTK_CAN_FOCUS);
   gtk_widget_show (PrevPage);
-  gtk_box_pack_start (GTK_BOX (vbox2), PrevPage, FALSE, TRUE, 3);
+  gtk_box_pack_start (GTK_BOX (CmdVBoxLeft), PrevPage, FALSE, TRUE, 3);
   gtk_signal_connect (GTK_OBJECT (PrevPage), "clicked",
                       GTK_SIGNAL_FUNC (on_GotoPage_clicked), "b"); 
   gtk_tooltips_set_tip(gtk_tooltips_new(), PrevPage,
@@ -347,7 +342,7 @@ create_xgdvi ()
   if (no_focus == 1)
     GTK_WIDGET_UNSET_FLAGS(NextPage, GTK_CAN_FOCUS);
   gtk_widget_show (NextPage);
-  gtk_box_pack_start (GTK_BOX (vbox2), NextPage, FALSE, TRUE, 3);
+  gtk_box_pack_start (GTK_BOX (CmdVBoxLeft), NextPage, FALSE, TRUE, 3);
   gtk_signal_connect (GTK_OBJECT (NextPage), "clicked",
                       GTK_SIGNAL_FUNC (on_GotoPage_clicked), " "); 
   gtk_tooltips_set_tip(gtk_tooltips_new(), NextPage,
@@ -358,7 +353,7 @@ create_xgdvi ()
   if (no_focus == 1)
     GTK_WIDGET_UNSET_FLAGS(LastPage, GTK_CAN_FOCUS);
   gtk_widget_show (LastPage);
-  gtk_box_pack_start (GTK_BOX (vbox2), LastPage, FALSE, TRUE, 3);
+  gtk_box_pack_start (GTK_BOX (CmdVBoxLeft), LastPage, FALSE, TRUE, 3);
   gtk_signal_connect (GTK_OBJECT (LastPage), "clicked",
                       GTK_SIGNAL_FUNC (on_GotoPage_clicked), ">"); 
   gtk_tooltips_set_tip(gtk_tooltips_new(), LastPage,
@@ -367,13 +362,13 @@ create_xgdvi ()
 
   hsep = gtk_hseparator_new();
   gtk_widget_show (hsep);
-  gtk_box_pack_start (GTK_BOX (vbox2), hsep, FALSE, TRUE, 5);
+  gtk_box_pack_start (GTK_BOX (CmdVBoxLeft), hsep, FALSE, TRUE, 5);
 
   JumpToPage = gtk_button_new_with_label (_("Go"));
   if (no_focus == 1)
     GTK_WIDGET_UNSET_FLAGS(JumpToPage, GTK_CAN_FOCUS);
   gtk_widget_show (JumpToPage);
-  gtk_box_pack_start (GTK_BOX (vbox2), JumpToPage, FALSE, TRUE, 3);
+  gtk_box_pack_start (GTK_BOX (CmdVBoxLeft), JumpToPage, FALSE, TRUE, 3);
   gtk_signal_connect (GTK_OBJECT (JumpToPage), "clicked",
                       GTK_SIGNAL_FUNC (on_JumpToPage_clicked), NULL); 
   gtk_tooltips_set_tip(gtk_tooltips_new(), JumpToPage,
@@ -397,7 +392,7 @@ create_xgdvi ()
     ( gtk_clist_get_hadjustment(GTK_CLIST(PageList)),
       gtk_clist_get_vadjustment(GTK_CLIST(PageList)));
   gtk_widget_show (PageListScroll);
-  gtk_box_pack_start (GTK_BOX (vbox2), PageListScroll, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (CmdVBoxLeft), PageListScroll, TRUE, TRUE, 0);
   gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (PageListScroll),
 					 PageList);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (PageListScroll),
@@ -411,13 +406,13 @@ create_xgdvi ()
 
   hsep = gtk_hseparator_new();
   gtk_widget_show (hsep);
-  gtk_box_pack_start (GTK_BOX (vbox2), hsep, FALSE, TRUE, 5);
+  gtk_box_pack_start (GTK_BOX (CmdVBoxLeft), hsep, FALSE, TRUE, 5);
 
   License = gtk_button_new_with_label (_("License..."));
   if (no_focus == 1)
     GTK_WIDGET_UNSET_FLAGS(License, GTK_CAN_FOCUS);
   gtk_widget_show (License);
-  gtk_box_pack_start (GTK_BOX (vbox2), License, FALSE, TRUE, 3);
+  gtk_box_pack_start (GTK_BOX (CmdVBoxLeft), License, FALSE, TRUE, 3);
   gtk_signal_connect (GTK_OBJECT (License), "clicked",
                       GTK_SIGNAL_FUNC (on_License_clicked), NULL);
   gtk_tooltips_set_tip(gtk_tooltips_new(), License,
@@ -428,7 +423,7 @@ create_xgdvi ()
   if (no_focus == 1)
     GTK_WIDGET_UNSET_FLAGS(Exit, GTK_CAN_FOCUS);
   gtk_widget_show (Exit);
-  gtk_box_pack_start (GTK_BOX (vbox2), Exit, FALSE, TRUE, 3);
+  gtk_box_pack_start (GTK_BOX (CmdVBoxLeft), Exit, FALSE, TRUE, 3);
   gtk_signal_connect (GTK_OBJECT (Exit), "clicked",
                       GTK_SIGNAL_FUNC (on_Exit_clicked), NULL);
   gtk_tooltips_set_tip(gtk_tooltips_new(), Exit,
@@ -481,11 +476,12 @@ create_xgdvi ()
 gint
 event_key_press(GtkWidget *w, GdkEventKey *ev)
 {
-  char cmd;
+  int  cmd;
 
   if (ev->length > 1)
     return TRUE;
 
+  cmd = -1;
   if (ev->length == 0){
     switch(ev->keyval){
     default:  return TRUE;
@@ -497,6 +493,8 @@ event_key_press(GtkWidget *w, GdkEventKey *ev)
     case GDK_Page_Down:  cmd = CTL('V'); break;
     case GDK_Home:       cmd = '<';      break;
     case GDK_End:        cmd = '>';      break;
+    case GDK_F1:         change_button_mode(1); break;
+    case GDK_F2:         change_button_mode(0); break;
     }
   } else if (ev->length == 1){
     cmd = ev->string[0];
@@ -652,7 +650,7 @@ update_page_display(void)
     return;
 
   vadj = gtk_scrolled_window_get_vadjustment
-    (GTK_SCROLLED_WINDOW(PageListScroll));
+           (GTK_SCROLLED_WINDOW(PageListScroll));
   p = (double)page / (double)dvi->pages; 
   p1 = ((double) page - 2) / (double)dvi->pages; 
   p2 = ((double) page + 2) / (double)dvi->pages; 
