@@ -37,7 +37,7 @@
 #include <X11/Xutil.h>
 #include <X11/cursorfont.h>
 
-#include "dvi-2_6.h"
+#include "libdvi29.h"
 #include "defs.h"
 #include "cf-xldvi.h"
 #include "resource.h"
