@@ -35,9 +35,9 @@
 #include <Xm/Xm.h>
 
 #include "cf-xmdvi.h"
-#include "dvi-2_6.h"
-#include "dev.h"
+#include "libdvi29.h"
 #include "defs.h"
+#include "dev.h"
 #include "resource.h"
 #include "ui.h"
 #include "window.h"
