@@ -60,9 +60,9 @@
 #include <Xm/MessageB.h>
 #include <Xm/Text.h>
 
-#include "cf-xmdvi.h"
-#include "dvi-2_6.h"
+#include "libdvi29.h"
 #include "defs.h"
+#include "cf-xmdvi.h"
 #include "resource.h"
 #include "window.h"
 #include "dev.h"
