@@ -45,10 +45,10 @@
 #include <Xm/Xm.h>
 #include <Xm/Separator.h>
 
-#include "cf-xmdvi.h"
-#include "dvi-2_6.h"
+#include "libdvi29.h"
 #include "dev.h"
 #include "defs.h"
+#include "cf-xmdvi.h"
 #include "resource.h"
 #include "paper.h"
 #include "window.h"
